"""Access control exceptions."""


class Error(Exception):

    """Base exception class.

    All errors extend this class which is useful to dispatch in error handlers.
    """


class PermissionNotFound(Error):

    """Permission is not found.

    Can't filter objects because there's no permission that implements filtering.
    """


class RoleNotFound(Error):

    """Role is not found.

    Role is not found for the name returned by the get_role callback.
    """



"""Access control role."""

from balrog import exceptions


class Role(object):

    """Role, a set of permissions that identity can have access to."""

    def __init__(self, name, permissions):
        """Create a role.

        :param name: Unique role name within one policy.
        :param permissions: Permissions of the role.
        """
        self.name = name
        self.permissions = {}

        for permission in permissions:
            assert permission.name not in self.permissions, (
                'The permission `{0}` is already registered within this role.'.format(permission.name)
            )
            self.permissions[permission.name] = permission

    def check(self, identity, permission, *args, **kwargs):
        """Check if the identity has requested permission.

        :param identity: Currently authenticated identity.
        :param permission: Permission name.

        :return: True if identity role has this permission.
        """
        try:
            permission = self.permissions[permission]
        except KeyError:
            return False
        else:
            return permission.check(identity, *args, **kwargs)

    def filter(self, identity, permission, objects, *args, **kwargs):
        """Filter objects according to the permission this identity has.

        :param identity: Currently authenticated identity.
        :param permission: Permission name.
        :param objects: Objects to filter out.

        :returns: Filtered objects.
        :raises: `PermissionNotFound` when no permission is found that can
            filter the objects.
        """
        try:
            permission = self.permissions[permission]
        except KeyError:
            raise exceptions.PermissionNotFound()
        else:
            return permission.filter(identity, objects, *args, **kwargs)



"""Access control policy."""

from balrog import exceptions


class Policy(object):

    """Controls the access of a certain actor to a certain action on a resource."""

    def __init__(self, roles, get_identity, get_role):
        """Create and configure access control.

        :param roles: All roles of this access control.
        :param get_identity: Callable that returns the currently authenticated
            identity.
        :param get_role: Callable that returns role name of the currently authenticated
            identity.
        """
        self._get_identity = get_identity
        self._get_role = get_role
        self.roles = {}
        self.permissions = {}

        for role in roles:
            assert role.name not in self.roles, (
                u'The role `{0}` is already registered.'.format(role.name)
            )
            self.roles[role.name] = role
            for name, permission in role.permissions.items():
                if name not in self.permissions:
                    self.permissions[name] = permission

    def _check_permission(self, permission):
        """Check if the given permission exists in the list of permissions.

        :param permission: The permission to check.

        :raises: `PermissionNotFound` if the permissions wasn't found.
        """
        if self.permissions and permission not in self.permissions:
            raise exceptions.PermissionNotFound(
                'Permission {0} was not found in the list of all permissions.'.format(permission)
            )

    def get_identity(self, *args, **kwargs):
        """Get current identity.

        :returns: An identity object which can be provided via a callback.
        """
        return self._get_identity(*args, **kwargs)

    def get_role(self, identity, *args, **kwargs):
        """Get identity role.

        :returns: Identity role object which name can be provided via a callback.
        :raises: `RoleNotFound` if no role found for this identity.
        """
        name = self._get_role(identity, *args, **kwargs)

        try:
            return self.roles[name]
        except KeyError:
            raise exceptions.RoleNotFound(name)

    def check(self, permission, *args, **kwargs):
        """Check if the identity has requested permission.

        :param permission: Permission name.
        :return: `True` if identity role has this permission.
        :raises: `RoleNotFound` if no role was found.
        :raises: `PermissionNotFound` when no permission is found.
        """
        self._check_permission(permission)
        identity = self.get_identity(*args, **kwargs)
        role = self.get_role(identity, *args, **kwargs)
        return role.check(identity, permission, *args, **kwargs)

    def filter(self, permission, objects, *args, **kwargs):
        """Filter objects according to the permission this identity has.

        :param permission: Permission name.
        :param objects: Objects to filter out.
        :returns: Filtered objects.
        :raises: `RoleNotFound` if no role found for this identity.
        :raises: `PermissionNotFound` when no permission is found that can
            filter the objects.
        """
        self._check_permission(permission)
        identity = self.get_identity(*args, **kwargs)
        role = self.get_role(identity, *args, **kwargs)
        return role.filter(identity, permission, objects, *args, **kwargs)



"""Access control permission."""


class Permission(object):

    """Permission that identity can have."""

    def __init__(self, name):
        """Create permission.

        :param name: Unique permission name within one role.
        """
        self.name = name

    def check(self, identity, *args, **kwargs):
        """Check if permission applies to this identity.

        :param identity: Currently authenticated identity.

        :return: `True` if permission is granted to requested identity.
        :note: This function could be overridden in order to do additional check
            in the certain context.
        """
        return True

    def filter(self, identity, objects, *args, **kwargs):
        """Filter objects according to this permission and identity.

        :param identity: Currently authenticated identity.
        :param objects: Objects to filter out.

        :returns: Filtered objects.
        :note: This function should be overridden in order to implement the
            filtering in the certain context.
        """
        return objects



#!/usr/bin/env python
"""Balrog library setup file."""

import os
import codecs

from setuptools import setup

import balrog


long_description = []

for text_file in ['README.rst', 'CHANGES.rst']:
    with codecs.open(os.path.join(os.path.dirname(os.path.abspath(__file__)), text_file), encoding="utf-8") as f:
        long_description.append(f.read())

setup(
    name="balrog",
    description="Python access control library.",
    long_description='\n'.join(long_description),
    author="Paylogic International",
    license="MIT license",
    author_email="developers@paylogic.com",
    url="https://github.com/paylogic/balrog",
    version=balrog.__version__,
    classifiers=[
        "Development Status :: 6 - Mature",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS :: MacOS X",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3"
    ] + [("Programming Language :: Python :: %s" % x) for x in "2.6 2.7 3.4".split()],
    packages=["balrog"],
    tests_require=["tox"],

)



"""Balrog public API."""

__version__ = '1.1.0'

try:
    from balrog.policy import Policy
    from balrog.role import Role
    from balrog.permission import Permission
    from balrog.exceptions import Error, PermissionNotFound, RoleNotFound

    __all__ = ['Policy', 'Role', 'Permission', 'Error', 'PermissionNotFound', 'RoleNotFound']
except ImportError:
    # avoid import errors when only __version__ is needed (for setup.py)
    pass
